/*
 * Version 0.2.0
 * @authors: Nathan Green <NathanGreen.me@gmail.com>
 */


// Dynamic hyperlink common functions
//////////////////////////////////////////////////
kettos.widgets.hyperlinkTextSizeCommon_ = function(textElem, defaultSize, dragger, div, widgetResize) {
    /* Font Size */
    var getSliderValue = function() {
        var size = div.getAttribute("data-size") || defaultSize;
        return parseInt(size);
    };

    var setSliderValue = function(size) {
        div.setAttribute('data-size', size);
        textElem.style.fontSize = size;

        if (widgetResize) {
            div.parentNode.style.height = parseInt(size) + 2;
            div.parentNode.style.width = textElem.offsetWidth;

            dragger.dispatchEvent({
                type: goog.ui.Resizable.EventType.RESIZE,
                size: {
                    width: div.offsetWidth,
                    height: size
                }
            });
        }
    };

    var appearanceBlock = new kettos.menu.Block("Appearance");

    appearanceBlock.add21Row(
        kettos.menu.item.slider(dragger, getSliderValue, setSliderValue, "Adjust Text Size", 12, 40, .5),
        kettos.menu.item.button(
            dragger,
            function() {
                setSliderValue(defaultSize);
                kettos.menu.update();
            },
            function() {
                //code not required
            },
            'Reset'));

    return appearanceBlock;
}


kettos.widgets.hyperlinkRebuiltLinkCommon_ = function(textElem, defaultPage, dragger, div, widgetResize) {
    /* HYPERLINK TEXT */
    updateHyperlinkText = function() {
        var pageName = div.getAttribute("data-page") || defaultPage;
        var linkFormat = div.getAttribute("data-link-format") || "complete";
        var linkText;

        switch (linkFormat) {
            case ("none"):
                linkText = "";
                break;
            case ("ip"):
                linkText = "192.168.92.###";
                break;
            case ("short"):
                linkText = "192.168.92.### (" + pageName + ")";
                break;
            case ("complete"):
                linkText = "https://192.168.92.###/slate/designer/servlet?page=" + pageName;
                break;
            case ("default"):
                linkText = "https://192.168.92.###/slate/designer/servlet?page=" + pageName;
        }

        textElem.innerText = linkText;
        if (widgetResize) {
            div.parentNode.style.width = textElem.offsetWidth;
        }
    }

    if (textElem) {
        updateHyperlinkText();
    }
}


kettos.widgets.hyperlinkTextContentCommon_ = function(textElem, defaultPage, dragger, div, allowHide, widgetResize) {
    var contentBlock = new kettos.menu.Block("Content");

    contentBlock.addWideRow(goog.dom.createDom("span", null, "Select how the hyperlink address will be displayed"));

    div.getAttribute("data-link-format") === null ? div.setAttribute("data-link-format", "complete") : false;


    /* HYPERLINK FORMAT */
    if (allowHide) {
        contentBlock.addWideRow([kettos.menu.item.radio(dragger, function() {
            if (div.getAttribute("data-link-format") == "none") {
                return true;
            }
        }, function() {
            div.setAttribute("data-link-format", "none");
            kettos.widgets.hyperlinkRebuiltLinkCommon_(textElem, defaultPage, dragger, div, widgetResize);
        }, null, null, {}), goog.dom.createDom("span", null, "None")]);
    }

    contentBlock.addWideRow([kettos.menu.item.radio(dragger, function() {
        if (div.getAttribute("data-link-format") == "ip") {
            return true;
        }
    }, function() {
        div.setAttribute("data-link-format", "ip");
        kettos.widgets.hyperlinkRebuiltLinkCommon_(textElem, defaultPage, dragger, div, widgetResize);
    }, null, null, {}), goog.dom.createDom("span", null, "IP Only")]);

    contentBlock.addWideRow([kettos.menu.item.radio(dragger, function() {
        if (div.getAttribute("data-link-format") == "short") {
            return true;
        }
    }, function() {
        div.setAttribute("data-link-format", "short");
        kettos.widgets.hyperlinkRebuiltLinkCommon_(textElem, defaultPage, dragger, div, widgetResize);
    }, null, null, {}), goog.dom.createDom("span", null, "Short")]);

    contentBlock.addWideRow([kettos.menu.item.radio(dragger, function() {
        if (div.getAttribute("data-link-format") == "complete") {
            return true;
        }
    }, function() {
        div.setAttribute("data-link-format", "complete");
        kettos.widgets.hyperlinkRebuiltLinkCommon_(textElem, defaultPage, dragger, div, widgetResize);
    }, null, null, {}), goog.dom.createDom("span", null, "Complete")]);

    kettos.widgets.hyperlinkRebuiltLinkCommon_(textElem, defaultPage, dragger, div, widgetResize);

    return contentBlock;
}


kettos.widgets.hyperlinkTargetPageCommon_ = function(textElem, defaultPage, dragger, div, widgetResize) {
    var actionBlock = new kettos.menu.Block("Action");

    actionBlock.addWideRow(
        kettos.menu.item.input(
            dragger,
            function() {
                return div.getAttribute("data-page") || "";
            },
            function(value) {
                value = value.replace(/\s/g, '');
                div.setAttribute("data-page", value);
                let pageName = value || defaultPage
                kettos.widgets.hyperlinkRebuiltLinkCommon_(textElem, defaultPage, dragger, div, widgetResize);
            },
            "Target Page"));
    actionBlock.rows[0].children[0].getElementsByTagName("input")[0].setAttribute("placeholder", defaultPage);

    return actionBlock;
}


kettos.widgets.hyperlinkBrokenMessageCommon_ = function(defaultMessage, dragger, div) {
    var alternateBlock = new kettos.menu.Block("Alternative");

    alternateBlock.addWideRow(
        kettos.menu.item.input(
            dragger,
            function() {
                return div.getAttribute("data-broken-message") || "";
            },
            function(value) {
                div.setAttribute("data-broken-message", value);
            },
            "Message when broken"));
    alternateBlock.rows[0].children[0].getElementsByTagName("input")[0].setAttribute("placeholder", defaultMessage);

    return alternateBlock;
}


// Dynamic Link Button
//////////////////////////////////////////////////
/** @constructor */
kettos.widgets.DynamicLinkButton = function() {
    goog.base(this, "Dynamic Link Button", "DynamicLinkButton");
    this.categories = ["NETWORK"];
}

goog.inherits(kettos.widgets.DynamicLinkButton, kettos.widgets.Base);


kettos.widgets.DynamicLinkButton.prototype.createDragElement = function() {
    return this.dragElementHelper(100, 26, goog.dom.createDom("button", {
        "type": "button",
        "class": "full"
    }, "Hyperlink"));
}


kettos.widgets.DynamicLinkButton.setCommonMenuItems_ = function(dragger, div, content) {
    var actionBlock = kettos.widgets.hyperlinkTargetPageCommon_(null, "Main", dragger, div, false);

    content.add(actionBlock);
    content.addButtonStyling();
    content.addTooltip();
    content.addPositionAndSize();
    content.addClass();
}


kettos.widgets.DynamicLinkButton.prototype.getMenuItems = function(dragger, div) {
    var button = div.firstChild;
    var content = new kettos.menu.Content(dragger, div);
    var contentBlock = new kettos.menu.Block("Content");

    contentBlock.addWideRow(
        kettos.menu.item.input(
            dragger,
            kettos.widgets.make.getHtml(button),
            kettos.widgets.make.setHtml(button),
            "Label"));

    content.addBinding();
    content.add(contentBlock);
    kettos.widgets.DynamicLinkButton.setCommonMenuItems_(dragger, div, content);

    return content;
}

kettos.dict.register(new kettos.widgets.DynamicLinkButton());


// Dynamic Link Area
//////////////////////////////////////////////////
/** @constructor */
kettos.widgets.DynamicLinkArea = function() {
    goog.base(this, "Dynamic Link Area", "DynamicLinkArea");
    this.categories = ["NETWORK"];
}

goog.inherits(kettos.widgets.DynamicLinkArea, kettos.widgets.Base);


kettos.widgets.DynamicLinkArea.prototype.createDragElement = function() {
    var innerSpan = goog.dom.createDom("span", {
        "style": "position: absolute; width: inherit; height: auto; color: rgb(0, 119, 204); text-decoration: underline; cursor: pointer; z-index: 5000;",
        "innerText": ""
    });

    return kettos.widgets.addBorderAndFillCss_(this.dragElementHelper(300, 200, innerSpan, {
        class: kettos.constants.container
    }));
}


kettos.widgets.DynamicLinkArea.prototype.getMenuItems = function(dragger, div) {
    const defaultPage = "Main";
    var span = div.firstChild;
    var defaultSize = 16;
    var borderWidth = parseInt(div.style.borderWidth) || 0;
    var content = new kettos.menu.Content(dragger, div);
    var yOffset = parseInt(div.getAttribute('data-offset-y')) || 0;
    var xOffset = parseInt(div.getAttribute('data-offset-x')) || 0;

    span.style.width = div.offsetWidth - (2 * parseInt(div.style.borderWidth)) - (2 * xOffset) + "px";
    span.style.left = xOffset;
    span.style.fontSize = span.style.fontSize || defaultSize;
    div.getAttribute("data-link-format") === null ? div.setAttribute("data-link-format", "none") : false;

    var setVPos = function(verticalAlign) {
        switch (verticalAlign) {
            case "t":
                span.style.top = borderWidth + yOffset + "px";
                span.style.bottom = "auto";
                break;
            case "m":
                span.style.top = parseInt((div.offsetHeight - span.offsetHeight) / 2) + "px";
                span.style.bottom = "auto";
                break;
            case "b":
                span.style.top = "auto";
                span.style.bottom = borderWidth + yOffset + "px";
                break;
            default:
                span.style.top = borderWidth + yOffset + "px";
                span.style.bottom = "auto";
        }
    }

    var setHPos = function(horizontalAlign) {
        switch (horizontalAlign) {
            case "l":
                span.style.textAlign = "left";
                break;
            case "c":
                span.style.textAlign = "center";
                break;
            case "r":
                span.style.textAlign = "right";
                break;
            default:
                span.style.textAlign = "left"
        }
    }

    var actionBlock = kettos.widgets.hyperlinkTargetPageCommon_(span, "Main", dragger, div, false);
    var alternateBlock = kettos.widgets.hyperlinkBrokenMessageCommon_("Broken Link", dragger, div);
    var contentBlock = kettos.widgets.hyperlinkTextContentCommon_(span, "Main", dragger, div, false);
    var appearanceBlock = kettos.widgets.hyperlinkTextSizeCommon_(span, defaultSize, dragger, div, false);


    /* Vertical Alignment */
    appearanceBlock.addWideRow(
        kettos.menu.item.select(
            dragger,
            function() {
                let currentValue = div.getAttribute("data-v-align");
                setVPos(currentValue)
                return currentValue;
            },
            function(value) {
                div.setAttribute("data-v-align", value);
                setVPos(value);
                kettos.menu.update();
            },
            "Vertical Alignment", ["Top", "Middle", "Bottom"], ["t", "m", "b"]));

    /* Horizontal Alignment */
    appearanceBlock.addWideRow(
        kettos.menu.item.select(
            dragger,
            function() {
                let currentValue = div.getAttribute("data-h-align");
                setHPos(currentValue)
                return div.getAttribute("data-h-align")
            },
            function(value) {
                div.setAttribute("data-h-align", value);
                setHPos(value);
                kettos.menu.update();
            },
            "Horizontal Alignment", ["Left", "Center", "Right"], ["l", "c", "r"]));

    var contentBlock = kettos.widgets.hyperlinkTextContentCommon_(span, "Main", dragger, div, true, false);

    appearanceBlock.addRow(
        kettos.menu.item.input(
            dragger,
            function() {
                return div.getAttribute('data-offset-y') || '0'
            },
            function(value) {
                div.setAttribute('data-offset-y', parseInt(value));
            },
            'y-offset', {
                'type': 'number',
                'min': 0
            }),

        kettos.menu.item.input(
            dragger,
            function() {
                return div.getAttribute('data-offset-x') || '0'
            },
            function(value) {
                div.setAttribute('data-offset-x', parseInt(value));
            },
            'x-offset', {
                'type': 'number',
                'min': 0
            }),
        "Text Indent");

    content.addDescription([goog.dom.createDom("div", null, 'This widget is used to navigate to a local designer/built-in or an external web page entered in the "reference (hyperlink)" property.'), goog.dom.createDom("li", {
        style: "padding-top: 10px;"
    }, "Designer page - page name is case sensitive and should not have any spaces"), goog.dom.createDom("li", null, 'Built-in page - enter a "/" preceding the page name; page name is also case sensitive'), goog.dom.createDom("li", null, "External page - enter the URL; this assumes the network was setup for access to the network the URL resides on.")]);

    content.addBinding();
    content.addFill(div, 'Fill');
    content.addBorder(true);
    content.add(actionBlock);
    content.add(contentBlock);
    content.add(alternateBlock);
    content.add(appearanceBlock);
    content.addTooltip();
    content.addPositionAndSize();
    content.addClass();

    return content;
}

kettos.dict.register(new kettos.widgets.DynamicLinkArea());


// Dynamic Link Text
//////////////////////////////////////////////////
/** @constructor */
kettos.widgets.DynamicLinkText = function() {
    goog.base(this, "Dynamic Link Text", "DynamicLinkText");
    this.categories = ['NETWORK'];
}

goog.inherits(kettos.widgets.DynamicLinkText, kettos.widgets.Base);


kettos.widgets.DynamicLinkText.prototype.createDragElement = function() {
    return this.dragElementHelper(395, 18, goog.dom.createDom('font', {
        "style": "position: absolute; width: auto; height: auto; color: rgb(0, 119, 204); text-decoration: underline; white-space: nowrap; cursor: pointer;"
    }, 'https://192.168.92.###/slate/designer/servlet?page=Main'));
}


kettos.widgets.DynamicLinkText.prototype.getMenuItems = function(dragger, div) {
    var content = new kettos.menu.Content(dragger, div);
    var span = div.firstChild;
    const defaultPage = "Main";
    const defaultSize = 16;

    span.style.fontSize = span.style.fontSize || defaultSize;

    var actionBlock = kettos.widgets.hyperlinkTargetPageCommon_(span, defaultPage, dragger, div, false);
    var contentBlock = kettos.widgets.hyperlinkTextContentCommon_(span, defaultPage, dragger, div, false, true);
    var appearanceBlock = kettos.widgets.hyperlinkTextSizeCommon_(span, defaultSize, dragger, div, true);

    content.addBinding();
    content.add(actionBlock);
    content.add(contentBlock);
    content.add(appearanceBlock);
    content.addBackgroundStyling();
    content.addTooltip();
    content.addPositionAndSize();
    content.addClass();

    return content;
}

kettos.dict.register(new kettos.widgets.DynamicLinkText());


// IP Octet PassThru
//////////////////////////////////////////////////
/** @constructor */
kettos.widgets.IPPassThru = function() {
    goog.base(this, 'IP PassThru', 'IPPassThru');
    this.categories = ['NETWORK'];
}

goog.inherits(kettos.widgets.IPPassThru, kettos.widgets.Base);

kettos.widgets.IPPassThru.prototype.createDragElement = function() {
    var input = goog.dom.createDom('input', {
        'type': 'text',
        'class': 'full'
    });
    input.setAttribute('readonly', 'readonly');
    return this.dragElementHelper(150, 20, input, {'data-visible': 'off'});
}

kettos.widgets.IPPassThru.prototype.getMenuItems = function(dragger, div) {
    var content = new kettos.menu.Content(dragger, div);

    var setVisibility = function(value) {
        value === "on" ? div.style.opacity = 1 : div.style.opacity = .5;
    }

    var appearanceBlock = new kettos.menu.Block('Appearance');
    appearanceBlock.addWideRow(
        kettos.menu.item.checkbox(
            dragger,
            function() {
                var value = div.getAttribute('data-visible');
                setVisibility(value);
                return value;
            },
            function(value) {
                div.setAttribute('data-visible', value);
                setVisibility(value);
            },
            'Display this widget',
            true,
            null,
            'When checked, this widget will be visible in the Live environment; otherwise it will be hidden'));

    var informationBlock = new kettos.menu.Block('Information');

    informationBlock.addWideRow(goog.dom.createDom("span", null, "Use this widget to force the last octet " +
        "of the Base Module's IP address into another register"));

    informationBlock.addWideRow(
        kettos.menu.item.text('Example', "font-weight: bolder;")
    );

    informationBlock.addWideRow(
        kettos.menu.item.text("'192.168.92.10' ─► '10' ") //Alt+196 & Alt+16
    );

    content.addBinding();
    content.add(informationBlock);
    content.add(appearanceBlock);
    content.addInputStyling();
    content.addFormatting();
    content.addTooltip();
    content.addPositionAndSize();
    content.addClass();

    return content;
}

kettos.dict.register(new kettos.widgets.IPPassThru());