/*
 * Version 0.2.0
 * @authors: Nathan Green <NathanGreen.me@gmail.com>
 */


// Dynamic Links Common
//////////////////////////////////////////////////
const activeDeviceIP = (function() { // IP address of device being viewed
    let href = window.location.href; // complete page address
    return href.split("//")[1].split("/")[0]; // e.g., 192.168.92.10
})();

function makeDynamicIp(lastOctet) {
    var octets = activeDeviceIP.split("."); // e.g., (4) ["192", "168", "92", "10"]
    var octet4 = octets.pop(); // remove last octet --> Date: 02/06/2024 | Desc: Unused
    var network = octets.toString().replace(/,/g, "."); // e.g, 192.168.92
    var defaultNetwork = "192.168.92";
    network.length === 0 ? network = defaultNetwork : network = network; // Preview mode
    return network + "." + parseInt(lastOctet); // e.g., '192.168.92' + '.' + '20'
}

function makeShortAddress(lastOctet, pageName) {
    pageName = pageName || "Main";
    var newIpAddress = makeDynamicIp(lastOctet);
    var shortAddress = newIpAddress + " (" + pageName + ")";
    return shortAddress;
}

function makeFullAddress(lastOctet, pageName) {
    pageName = pageName || "Main";
    var newIpAddress = makeDynamicIp(lastOctet);
    var fullAddress = "https://" + newIpAddress + "/slate/designer/servlet?page=" + pageName;
    return fullAddress;
}


// Dynamic Link Button
//////////////////////////////////////////////////
kettos.live.DynamicLinkButton = function(div) {
    var page = div.getAttribute("data-page"); // target designer page --> e.g., "Main"
    var register = div.getAttribute("data-mr") || "m" + div.getAttribute("data-m") + "r" + div.getAttribute("data-r");
    var obj = {
        "id": register
    };
    div.firstChild.style.cursor = "pointer";

    goog.events.listen(div, 'click', kettos.live.net.getFuncOnChange({
        el: div,
        getValue: function(value) {
            var mr = div.getAttribute('data-mr');
            kettos.live.net.sendRequest("gv", [{
                id: mr
            }], function(response) {
                let r = response.r[0];
                let value = parseInt(r.v);
                console.log(makeFullAddress(value, page)); // https://192.168.92.20/slate/designer/servlet?page=Main
                if (kettos.live.local) {
                    new kettos.dialog.Alert('This type of link does not work in the preview mode.');
                } else {
                    window.location.href = makeFullAddress(value, page); // perform navigation
                }
            });
        }
    }));
}


// Dynamic Link Area
//////////////////////////////////////////////////
kettos.live.DynamicLinkArea = function(div) {
    var span = div.firstChild;
    var page = div.getAttribute("data-page"); // target designer page --> e.g., "Main"
    /*var register = div.getAttribute("data-mr") || "m" + div.getAttribute("data-m") + "r" + div.getAttribute("data-r");
    var obj = {
        "id": register
    };*/ // --> Date: 02/06/2024 | Desc: removed because not used
    div.style.cursor = "pointer";

    /* HYPERLINK TEXT*/
    updateHyperlinkText = function() {
        var pageName = div.getAttribute("data-page") || defaultPage;
        var linkFormat = div.getAttribute("data-link-format") || "complete";
        var linkText;
    }

    kettos.live.net.listenForValues(div, function(value) {
        var linkFormat = div.getAttribute("data-link-format") || "complete";
        div.value = parseInt(value);
        switch (linkFormat) {
            case ("none"):
                linkText = "";
                break;
            case ("ip"):
                linkText = makeDynamicIp(value);
                break;
            case ("short"):
                linkText = makeShortAddress(value, page);
                break;
            case ("complete"):
                linkText = makeFullAddress(value, page);
                break;
            case ("default"):
                linkText = makeFullAddress(value, page);
        }
        span.innerText = linkText;
    });

    goog.events.listen(div, 'click', function() {
        console.log(makeFullAddress(div.value, page)); // https://192.168.92.20/slate/designer/servlet?page=Main
        if (kettos.live.local) {
            new kettos.dialog.Alert('This type of link does not work in the preview mode.');
        } else {
            window.location.href = makeFullAddress(div.value, page); // perform navigation
        }
    });
}


// Dynamic Link Text
//////////////////////////////////////////////////
kettos.live.DynamicLinkText = function(div) {
    goog.events.listen(div, 'click', function() {
        new kettos.dialog.Alert('Coming soon. . .');
    });
}


// IP PassThru
//////////////////////////////////////////////////
kettos.live.IPPassThru = function(div) {
    // Handle visibility
    if (div.getAttribute('data-visible') == "on") {
        div.style.opacity = 1; // make visible
    } else {
        div.style.visibility = "hidden"; // --> Date: 02/06/2024 | Desc: reduces code
        /*div.style.opacity = 0; // make invisible ("display: none" will remove kettos value updates)
        div.parentNode.style.height = 0;  // instead, height and width should be wet to '0'
        div.parentNode.style.width = 0;*/
        div.parentNode.style.zIndex = -1; // send to back
    }

    // Create change event to dispatch
    var event = document.createEvent("HTMLEvents");
    event.initEvent("change", false, true);

    // Get Value
    const getIPAddress = function(callback) {
        ipAddr = document.querySelector('[data-mr="m1r119"]').firstChild.value;
        let octet4 = (ipAddr).split(".").pop();
        callback(octet4);
    }

    // Set Value
    const setIPOctet = function(value) {
        value = parseInt(value);
        goog.events.listen(div, 'change', kettos.live.net.getFuncOnChange({ // This was disabled
             el: div,
             getValue: function() {
                 return div.firstChild.value;
             }
         }));

        if (div.firstChild.value != value) { // only update if new value is different from existing value
            console.log("Old value: " + div.firstChild.value, "|", "New value: " + value);
            div.firstChild.value = value; // update value
            div.dispatchEvent(event);
        }
    }

    // Listen for IP address of Base Module
    kettos.live.net.listenForValues(div, function(value) {
        div.firstChild.value = value; // assign first value to widget
        getIPAddress(setIPOctet); // IP address is always m1r119
    });
}